package il.ac.tau.cs.software1.gamesearch;

import java.util.Arrays;

/**
 * ConnectFourMove
 * 
 * Represents a board situation in the Connect Four game
 * 
 */
public class ConnectFourBoard {

	/**
	 * The representation of the board.
	 * 
	 * Representation invariant: board is a 6-by-7 array (array of arrays) whose
	 * entries are all 0, 1, or -1. The entries represent empty slots (-1),
	 * slots occupied by white pieces (0) or black pieces (1).
	 * 
	 */

	private int[][] board;

	private final int NUM_ROWS = 6; // a constant

	private final int NUM_COLS = 7; // a constant
	
	private final double WHITE_WON = 0.0; // a constant
	
	private final double BLACK_WON = 1.0; // a constant
	
	private final double DRAW = 0.5; // a constant
	
	
	/**
	 * Creates an empty board (initial state of the game).
	 * 
	 */

	public ConnectFourBoard() {
		board = new int[NUM_ROWS][NUM_COLS];

		for (int i = 0; i < NUM_ROWS; i++) {
			for (int j = 0; j < NUM_COLS; j++) {
				board[i][j] = -1;
			}
		}
	}

	/**
	 * 
	 * @param m
	 *            a move
	 * @param isWhite
	 *            true if m specifies a move for the white player, black
	 *            otherwise.
	 * @precondition m is a legal move
	 * @precondition if white_move==true, then the board's state should be such
	 *               that it is the White's turn, otherwise it should be the
	 *               black's turn.
	 * @precondition isGameOver()== false              
	 * @return a new board that represents the board after the move.
	 *
	 */
	
	public ConnectFourBoard move(ConnectFourMove m, boolean isWhite) {
		ConnectFourBoard newBoard = new ConnectFourBoard();
		for (int i = 0; i < NUM_ROWS; i++) {
			for (int j = 0; j < NUM_COLS; j++) {
				newBoard.board[i][j] = this.board[i][j];
			}
		}

		int col = m.column();
		int row = 0;
		for (; row < NUM_ROWS; row++) {
			if (board[row][col] == -1)
				break;
		}

		newBoard.board[row][col] = isWhite ? 0 : 1;
		return newBoard;
	}

	/**
	 * @precondition none
	 * @return A score that represents numerically the state of the game. The
	 *         returned value must be 0.0, 1.0, 0.5, or -1.0 A score of 0.0
	 *         means that the white won (has 4 consecutive pieces in a row,
	 *         column, or diagonally). A score of 1.0 means that the black won.
	 *         a score of 0.5 means that the game ended in a draw: the board is
	 *         full but neither black nor white won. A score of -1.0 means that
	 *         the game has not ended yet.
	 */
	public double isGameOver() {
      
	}
	

	/**
	 * @precondition none
	 * @return a score that represents numerically the state of the game. The
	 *         score should be between 0 and 1 - exclusive. 
	 *         (1 and 0 should not be returned in middle-game situations. 
	 *         Otherwise the search might persue a non-winning strategy. ) 
	 * 
	 * Scores near 0 should indicate an advantage to White, scores near 1 should
	 * indicate an advantage to Black. The strongest the position, the closest
	 * the score should be to 0 or 1.
	 */

	public double score() {

    } 

	/**
	 * @precondition: none
	 * @return an array of all the legal moves in this board position The array
	 *         must be empty if the game has ended (a draw or one of the players
	 *         won).
	 */

	public ConnectFourMove[] legalMoves() {
		if (isGameOver() != -1.0)
			return new ConnectFourMove[0];

		ConnectFourMove[] legalMoves = new ConnectFourMove[NUM_COLS];
		int j = 0;
		for (int i = 0; i < NUM_COLS; i++) {
			if (board[NUM_ROWS - 1][i] == -1)
				legalMoves[j++] = new ConnectFourMove(i);
		}
		if (j == NUM_COLS)
			return legalMoves;
		ConnectFourMove[] legalMoves2 = new ConnectFourMove[j];
		System.arraycopy(legalMoves, 0, legalMoves2, 0, j);
		return legalMoves2;
	}

	/**
	 * @precondition none
	 * @effect prints the current state of the board to System.out
	 * 
	 */
	public void print() {
		System.out.println();
		for (int i = NUM_ROWS - 1; i >= 0; i--) {
			System.out.print("row " + (i + 1) + ": ");
			for (int j = 0; j < NUM_COLS; j++) {
				System.out.print("|");
				char c;
				switch (board[i][j]) {
				case 0:
					c = 'W';
					break;
				case 1:
					c = 'B';
					break;
				default:// case -1:
					c = ' ';
					break;
				}
				System.out.print(c);
			}
			System.out.println("|");
		}
		System.out.println("       ---------------");
		System.out.print("        ");
		for (int j = 0; j < NUM_COLS; j++) {
			System.out.print(j + 1 + " ");
		}
		System.out.println();
	}

	
}
